#
# message.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@loveruby.net>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU General Public License version 2 or later.
#

module Delta

  class Message

    def initialize( win, adj )
      @win = win
      @adj = adj
    end

    def freeze
      if block_given? then
        begin
          @win.freeze
          yield
        ensure
          @win.thaw
        end
      else
        @win.freeze
      end
    end

    def thaw
      @win.thaw
    end

    def print( port )
      buf = mail_to_text(port)
      begin
        @win.freeze
        @win.set_point 0
        @win.forward_delete @win.get_length
        buf.each do |s|
          @win.insert_text s, @win.get_point if s
        end
      ensure
        @win.thaw
      end
    end

    def mail_to_text( port )
      m = TMail::Mail.new(port)
      buf = [ 'Date: ', m.header_string('date', '(none)'), "\n",
              'Subject: ', m.header_string('subject', ''), "\n" ]

      unless m.multipart?
        pushtext buf, m
      else
        m.parts.each do |part|
          pushtext buf, part
        end
      end

      buf
    end

    def pushtext( buf, mail )
      str = decode_mail(mail)
      unless mail.content_type('text/plain') == 'text/plain'
        buf.push "Content-Type: #{mail.header_string 'content-type',''}\n"
        buf.push "\n"
      end
      if mail.main_type('text') == 'text'
        buf.push "\n"
        buf.push NKF.nkf('-e', str)
      else
        buf.push "\n(skipped)\n\n"
      end
    end

    def decode_mail( mail )
      return '' unless mail.main_type('text') == 'text'
      mail.base64_decode
      if mail.type_param('type', '') == 'gzip'
        gzip_d(mail.body)
      else
        mail.body
      end
    end

    GZIP = 'gzip -dc'
    def gzip_d( str )
      open("| #{GZIP}", 'w+') {|f|
          f.write(str)
          f.close_write
          return f.read
      }
    end

    def read_ahead
      if @adj.value + @adj.page_size >= @adj.upper then
        false
      else
        # 90% ahead
        #@adj.value = [@adj.value + @adj.page_size * 9 / 10, @adj.upper - 1].min
        @adj.value = @adj.value + @adj.page_size * 9 / 10
        true
      end
    end

    def read_back
      if @adj.value <= @adj.lower then
        false
      else
        # 90% back
        #@adj.value = [@adj.value - @adj.page_size * 9 / 10, @adj.lower].max
        @adj.value = @adj.value - @adj.page_size * 9 / 10
        true
      end
    end
  
  end

end
